## $Id: test_browser.py 14721 2017-07-19 19:16:36Z henrik $
##
## Copyright (C) 2013 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Test the applicant-related UI components.
"""
import shutil
import tempfile
import datetime
import pytz
import os
from zope.component.hooks import setSite, clearSite
from zope.component import createObject, getUtility
from zope.catalog.interfaces import ICatalog
from zope.intid.interfaces import IIntIds
from zope.testbrowser.testing import Browser
from kofacustom.dspg.testing import FunctionalLayer
from waeup.kofa.app import University
from waeup.kofa.university.faculty import Faculty
from waeup.kofa.university.department import Department
from waeup.kofa.testing import FunctionalTestCase
from waeup.kofa.applicants.tests.test_batching import ApplicantImportExportSetup
from waeup.kofa.applicants.container import ApplicantsContainer

session = datetime.datetime.now().year - 2
ndftcontainer_name = u'ndft%s' % session

class ApplicantUITest(FunctionalTestCase):
    """Perform some browser tests.
    """
    layer = FunctionalLayer

    def setUp(self):
        super(ApplicantUITest, self).setUp()
        # Setup a sample site for each test
        app = University()
        self.dc_root = tempfile.mkdtemp()
        app['datacenter'].setStoragePath(self.dc_root)

        # Prepopulate the ZODB...
        self.getRootFolder()['app'] = app
        # we add the site immediately after creation to the
        # ZODB. Catalogs and other local utilities are not setup
        # before that step.
        self.app = self.getRootFolder()['app']
        # Set site here. Some of the following setup code might need
        # to access grok.getSite() and should get our new app then
        setSite(app)

        # Add ndft applicants container
        self.ndftcontainer = ApplicantsContainer()
        self.ndftcontainer.mode = 'create'
        self.ndftcontainer.code = ndftcontainer_name
        self.ndftcontainer.prefix = u'ndft'
        self.ndftcontainer.application_category = u'ndft'
        self.ndftcontainer.year = session
        self.ndftcontainer.application_fee = 300.0
        #self.ndftcontainer.title = u'This is the %s container' % ndftcontainer_name
        self.app['applicants'][ndftcontainer_name] = self.ndftcontainer

        delta = datetime.timedelta(days=10)
        self.ndftcontainer.startdate = datetime.datetime.now(pytz.utc) - delta
        self.ndftcontainer.enddate = datetime.datetime.now(pytz.utc) + delta

        # Populate university
        self.certificate = createObject('waeup.Certificate')
        self.certificate.code = 'CERT1'
        self.certificate.application_category = 'ndft'
        self.certificate.start_level = 100
        self.certificate.end_level = 500
        self.certificate.study_mode = u'nce_ft'
        self.app['faculties']['fac1'] = Faculty()
        self.app['faculties']['fac1']['dep1'] = Department()
        self.app['faculties']['fac1']['dep1'].certificates.addCertificate(
            self.certificate)

        # Add (customized) applicants
        ndftapplicant = createObject(u'waeup.Applicant')
        ndftapplicant.firstname = u'Anna'
        ndftapplicant.lastname = u'Post'
        self.app['applicants'][ndftcontainer_name].addApplicant(ndftapplicant)
        self.ndftapplication_number = ndftapplicant.application_number
        self.ndftapplicant = self.app['applicants'][ndftcontainer_name][
            self.ndftapplication_number]
        self.ndftapplicant_path = ('http://localhost/app/applicants/%s/%s'
            % (ndftcontainer_name, self.ndftapplication_number))

        self.browser = Browser()
        self.browser.handleErrors = False

    def tearDown(self):
        super(ApplicantUITest, self).tearDown()
        shutil.rmtree(self.dc_root)
        clearSite()
        return

    def fill_correct_values(self):
        self.browser.getControl(name="form.reg_number").value = '1234'
        self.browser.getControl(name="form.firstname").value = 'John'
        self.browser.getControl(name="form.lastname").value = 'Tester'
        self.browser.getControl(name="form.date_of_birth").value = '09/09/1988'
        self.browser.getControl(name="form.lga").value = ['foreigner']
        #self.browser.getControl(name="form.nationality").value = ['NG']
        #self.browser.getControl(name="form.sex").value = ['m']
        self.browser.getControl(name="form.email").value = 'xx@yy.zz'

    def test_manage_and_view_applicant(self):
        # Managers can manage ndft applicants.
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.ndftapplicant_path)
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertTrue("'O' Level" in self.browser.contents)
        self.assertFalse("Higher" in self.browser.contents)
        self.browser.open(self.ndftapplicant_path + '/manage')
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertTrue("'O' Level" in self.browser.contents)
        self.assertTrue("Higher" in self.browser.contents)
        self.browser.open(self.ndftapplicant_path + '/edit')
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertTrue("'O' Level" in self.browser.contents)
        self.assertFalse("Higher" in self.browser.contents)
        self.browser.open(self.ndftapplicant_path + '/application_slip.pdf')
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        # Now we turn the ndft applicant into an hndft applicant.
        self.ndftapplicant.applicant_id = u'hndft_anything'
        self.browser.open(self.ndftapplicant_path)
        self.assertTrue("Higher" in self.browser.contents)
        self.assertTrue("'O' Level" in self.browser.contents)
        self.browser.open(self.ndftapplicant_path + '/edit')
        self.assertTrue("Higher" in self.browser.contents)
        self.assertTrue("'O' Level" in self.browser.contents)

        self.browser.open(self.ndftapplicant_path + '/manage')
        # Manager can fill and save the form
        self.fill_correct_values()
        self.browser.getControl("Save").click()
        self.assertMatches('...Form has been saved...', self.browser.contents)
        return

class ApplicantExporterTest(ApplicantImportExportSetup):

    layer = FunctionalLayer

    def setUp(self):
        super(ApplicantExporterTest, self).setUp()
        self.outfile = os.path.join(self.workdir, 'myoutput.csv')
        self.cat = getUtility(ICatalog, name='applicants_catalog')
        self.intids = getUtility(IIntIds)
        return

    def setup_applicant(self, applicant):
        # set predictable values for `applicant`
        applicant.reg_number = u'123456'
        applicant.applicant_id = u'dp2011_654321'
        applicant.firstname = u'Anna'
        applicant.lastname = u'Tester'
        applicant.middlename = u'M.'
        applicant.nationality = u'NG'
        applicant.date_of_birth = datetime.date(1981, 2, 4)
        applicant.sex = 'f'
        applicant.email = 'anna@sample.com'
        applicant.phone = u'+234-123-12345'
        applicant.course1 = self.certificate
        applicant.course2 = self.certificate
        applicant.course_admitted = self.certificate
        applicant.notice = u'Some notice\nin lines.'
        applicant.screening_score = 98
        applicant.screening_venue = u'Exam Room'
        applicant.screening_date = u'Saturday, 16th June 2012 2:00:00 PM'
        applicant.password = 'any password'
        return applicant
